/*
 * Test pipe operations.
 *
 * Copyright (C) 2020, 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/re/env.h>
#include <l4/sys/err.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "file.h"
#include "memory_utils.h"



static void show(file_t *file, offset_t step, offset_t sample)
{
  /* Allocate a buffer for sampling from the file. */

  char buf[sample + 1];

  for (offset_t offset = 0; offset < file_populated_span(file); offset += step)
  {
    printf("show %ld of %ld...\n", offset, file_populated_span(file));

    unsigned long remaining = file_populated_span(file) - offset;
    unsigned long sample_remaining = remaining < sample ? remaining : sample;

    printf("%ld bytes from %p...\n", sample_remaining, (file->memory + offset));
    strncpy(buf, (file->memory + offset), sample_remaining);
    buf[sample_remaining] = '\0';
    printf("%s\n", buf);
  }
}



const unsigned int PIPE_PAGES = 2;

int main(void)
{
  /* Obtain access to the filesystem. */

  l4_cap_idx_t server = l4re_env_get_cap("server");

  /* Invoke the open method to receive the file reference. */

  file_t reader, writer;
  long err = pipe_open(page(PIPE_PAGES), &reader, &writer, server);

  if (err)
  {
    printf("Could not obtain pipe: %s\n", l4sys_errtostr(err));
    return 1;
  }

  /* Use the writer to fill the pipe with data. */

  for (int region = 0; region < 3; region++)
  {
    printf("Write %ld to pipe at %p...\n", file_span(&writer), writer.memory);

    memset(writer.memory, (int) 'a' + region, file_span(&writer));

    err = pipe_written(&writer, file_span(&writer));

    if (err)
    {
      printf("Written data error: %s\n", l4sys_errtostr(err));
      return 1;
    }

    show(&writer, page(1), 60);

    err = pipe_next(&writer);

    if (err)
    {
      printf("Region traversal error: %s\n", l4sys_errtostr(err));
      return 1;
    }
  }

  /* Use the reader to obtain data from the pipe. */

  err = pipe_current(&reader);

  while (!err)
  {
    printf("show...\n");
    show(&reader, page(1), 60);
    err = pipe_next(&reader);
  }

  if (err)
    printf("Reading termination condition: %s\n", l4sys_errtostr(err));

  printf("Data shown.\n");

  return 0;
}

// vim: tabstop=2 expandtab shiftwidth=2
